# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimDate,
    HprimForeign,
    HprimString,
    Segment,
)

logger = logging.getLogger("driverng.alegria")


class AlegriaSegmentH(HeaderSegment):
    sender_name = HprimString(5)
    version = HprimString(13, default="LIS2-A2")
    date_message = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class AlegriaSegmentQ(Segment):
    starting_range_id_number = HprimString(3)
    begin_request_result_date = HprimDate(4)
    end_request_result_date = HprimDate(5)


class AlegriaSegmentP(Segment):
    # Assigned by the laboratory
    laboratory_patient_id = HprimString(4)

    orders = HprimForeign("O", "patient")


class AlegriaSegmentO(Segment):
    sample_id = HprimString(3)
    local_code = HprimString((5, 4))
    # S: Stat
    # A: As soon as possible
    # R: Routine
    priority = HprimString(6)
    action_code = HprimString(12)
    # Used when QC should be run
    laboratory_field_number_1 = HprimString(21)
    report_types = HprimString(26)

    results = HprimForeign("R", "order")


class AlegriaSegmentR(Segment):
    test_id = HprimString((3, 4))
    value = HprimString((4, 1))
    unit = HprimString((4, 2))
    interpretation = HprimString((4, 3))
    result_status = HprimString(9)
    operator = HprimString(11)
    date_result_completed = HprimDate(13)


class AlegriaSegmentC(Segment):
    comment_source = HprimString(3)
    comment_text = HprimString(4)
    comment_type = HprimString(5)


class AlegriaSegmentL(Segment):
    terminaison_code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {"code": "alegria", "label": "ALEGRIA SEBIA"}
    SEGMENT_CLASS_MAP = {
        "H": AlegriaSegmentH,
        "Q": AlegriaSegmentQ,
        "P": AlegriaSegmentP,
        "O": AlegriaSegmentO,
        "R": AlegriaSegmentR,
        "C": AlegriaSegmentC,
        "L": AlegriaSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"
    OPTIONS = [
        AstmMessageV2.Option(
            "alegria_sender_name", label="Sender name (H.5)", default=""
        ),
        AstmMessageV2.Option("alegria_prefix_qc", label="Préfixe QC", default="QC"),
    ]

    def __init__(self, **kwargs):
        self.alegria_sender_name = kwargs.pop("alegria_sender_name", "")
        self.alegria_prefix_qc = kwargs.pop("alegria_prefix_qc", "QC")
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = AlegriaSegmentH(type_segment="H", **kwargs)
        message.end = AlegriaSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        message = self.create(
            date_message=context["now"], sender_name=self.alegria_sender_name
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            priority={"A": "A", "S": "S", worklist.ELSE: "R"},
            action={"del": worklist.IGNORE, worklist.ELSE: "N"},
        ):
            seg_p = message.patients.add(laboratory_patient_id=patient.patient_id)
            for sample, action, priority in patient.samples:
                for order in sample.orders:
                    seg_p.orders.add(
                        sample_id=sample.sample_id,
                        local_code=order["code"],
                        priority=priority,
                        action_code=action,
                        report_types="Q" if context["query"] else "O",
                    )
        if message.patients or (context["query"] and context["query"].queries):
            return message

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient", sample_id=seg_q.starting_range_id_number
                )
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    sample_id = seg_o.sample_id
                    if sample_id.startswith(self.alegria_prefix_qc):
                        sample = worklist.get_or_create(
                            type="control", control_lot=sample_id, sample_id=sample_id
                        )
                    else:
                        sample = worklist.get_or_create(
                            type="patient", sample_id=sample_id
                        )
                    for seg_r in seg_o.results:
                        comments = []
                        for seg_c in seg_r.commentaires:
                            comments.append(seg_c.comment_text)
                        # Resultat quantitafif
                        sample.results.add(
                            code=seg_r.test_id,
                            value=seg_r.value,
                            unit=seg_r.unit,
                            operator=seg_r.operator,
                            date=seg_r.date_result_completed,
                            comments=comments,
                        )
                        # Interpretation
                        if seg_r.interpretation:
                            sample.results.add(
                                code="{}I".format(seg_r.test_id),
                                value=seg_r.interpretation,
                                operator=seg_r.operator,
                                date=seg_r.date_result_completed,
                                comments=comments,
                            )
        return worklist
